// Vikunja is a to-do list application to facilitate your life.
// Copyright 2018-2021 Vikunja and contributors. All rights reserved.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public Licensee as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public Licensee for more details.
//
// You should have received a copy of the GNU Affero General Public Licensee
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package webhooks


import (

	"code.vikunja.io/api/pkg/log"

	"github.com/ThreeDotsLabs/watermill/message"
)


func GenerateMiddleware() message.HandlerMiddleware {
	runtimeConfig := ProcessConfig()
	return func(hf message.HandlerFunc) message.HandlerFunc {
		return func(msg *message.Message) ([]*message.Message, error) {
			topic := message.SubscribeTopicFromCtx(msg.Context() )
			handleOutgoingWebhook(runtimeConfig, topic, msg)
			newMsg, err := hf(msg)
			if err != nil {
				return nil, err
			}
			return newMsg, nil
		}
	}
}



func handleOutgoingWebhook(cfg []WebhookRuntimeConfig, topic string, msg *message.Message) {
	log.Debugf("Webhook handler for '%s'", topic)
	for i, entry := range cfg {
		log.Debugf("Checking webhook config [%d]", i)
		if entry.FilterFunc(topic) {
			log.Debugf("Executing webhook [%d]", i)
			entry.ExecuteFunc(topic, msg)
		}
	}
}

